@@ -1,6 +1,7 @@
 // Glaze Library
 // For the license information refer to glaze.hpp
 
+#include <cstdint>
 #include <random>
 #include <thread>
 
@@ -1132,6 +1133,19 @@ suite async_vector_tests = [] {
       expect(move_assigned.read()[2] == 3) << "Elements should be moved correctly";
    };
 
+   "compare"_test = [] {
+      glz::async_vector<int> v1;
+      v1.push_back(10);
+      v1.push_back(20);
+      v1.push_back(30);
+      glz::async_vector v2 = v1;
+
+      expect(v1 == v2);
+
+      std::vector<int> std_vec = {10, 20, 30};
+      expect(v1 == std_vec);
+   };
+
    "element_access"_test = [] {
       glz::async_vector<int> vec;
       vec.push_back(10);
@@ -1567,11 +1581,11 @@ suite additional_async_vector_tests = [] {
       threads.emplace_back([&]() {
          while (!stop) {
             try {
-               int sum = 0;
+               int64_t sum = 0; // use wider type to avoid overflow on fast machines
                {
                   auto proxy = vec.read();
                   for (auto it = proxy.begin(); it != proxy.end(); ++it) {
-                     sum += *it;
+                     sum += static_cast<int64_t>(*it);
                   }
                }
 
@@ -1629,7 +1643,10 @@ suite additional_async_vector_tests = [] {
             for (int i = 0; i < 100 && !stop; ++i) {
                try {
                   // Pick a random position
-                  size_t pos = std::rand() % (vec.size() + 1);
+                  thread_local std::mt19937 rng(std::random_device{}());
+                  size_t upper = vec.size();
+                  std::uniform_int_distribution<size_t> pos_dist(0, upper);
+                  size_t pos = pos_dist(rng);
                   {
                      auto proxy = vec.write();
                      auto insert_pos = proxy.begin();
@@ -1645,11 +1662,14 @@ suite additional_async_vector_tests = [] {
                   }
 
                   // Small delay to increase chance of contention
-                  std::this_thread::sleep_for(std::chrono::microseconds(std::rand() % 100));
+                  std::uniform_int_distribution<int> us_dist(0, 99);
+                  std::this_thread::sleep_for(std::chrono::microseconds(us_dist(rng)));
 
                   // Pick another random position for erasure
                   if (vec.size() > 0) {
-                     size_t erase_pos = std::rand() % vec.size();
+                     size_t cur = vec.size();
+                     std::uniform_int_distribution<size_t> erase_dist(0, cur - 1);
+                     size_t erase_pos = erase_dist(rng);
                      auto proxy = vec.write();
                      auto erase_iter = proxy.begin();
                      std::advance(erase_iter, erase_pos);
@@ -1687,7 +1707,10 @@ suite additional_async_vector_tests = [] {
             for (int i = 0; i < 100 && !stop; ++i) {
                try {
                   // Pick a random position
-                  size_t pos = std::rand() % (vec.size() + 1);
+                  thread_local std::mt19937 rng(std::random_device{}());
+                  size_t upper = vec.size();
+                  std::uniform_int_distribution<size_t> pos_dist(0, upper);
+                  size_t pos = pos_dist(rng);
                   {
                      auto proxy = vec.write();
                      auto insert_pos = proxy.begin();
@@ -1710,7 +1733,8 @@ suite additional_async_vector_tests = [] {
                   }
 
                   // Small delay to increase chance of contention
-                  std::this_thread::sleep_for(std::chrono::microseconds(std::rand() % 100));
+                  std::uniform_int_distribution<int> us_dist(0, 99);
+                  std::this_thread::sleep_for(std::chrono::microseconds(us_dist(rng)));
                }
                catch (const std::exception&) {
                   errors++;
@@ -1812,7 +1836,9 @@ suite additional_async_vector_tests = [] {
          auto it = proxy.begin();
          while (!stop) {
             try {
-               int jump = std::rand() % 20 - 10; // Random jump forward or backward
+               thread_local std::mt19937 rng(std::random_device{}());
+               std::uniform_int_distribution<int> jump_dist(-10, 9);
+               int jump = jump_dist(rng); // Random jump forward or backward
                if (it + jump >= proxy.begin() && it + jump < proxy.end()) {
                   it += jump;
                   it->access(); // Access the object to increment counter
@@ -1906,7 +1932,7 @@ suite additional_async_vector_tests = [] {
       });
 
       // Threads that read the vector during resizing
-      std::atomic<int> sum = 0;
+      std::atomic<uint64_t> sum = 0;
       for (int t = 0; t < 3; ++t) {
          threads.emplace_back([&]() {
             while (!stop) {
@@ -1914,7 +1940,7 @@ suite additional_async_vector_tests = [] {
                   size_t current_size = vec.size();
                   auto proxy = vec.read();
                   for (size_t i = 0; i < current_size && i < proxy.size(); ++i) {
-                     sum += proxy[i];
+                     sum.fetch_add(static_cast<uint64_t>(proxy[i]), std::memory_order_relaxed);
                   }
                }
                catch (const std::exception&) {
@@ -2261,4 +2287,49 @@ suite async_vector_json_tests = [] {
    };
 };
 
+suite async_vector_beve_tests = [] {
+   "async_vector write_beve / read_beve"_test = [] {
+      glz::async_vector<int> v;
+      v.push_back(1);
+      v.push_back(2);
+      v.push_back(3);
+      v.push_back(4);
+      v.push_back(5);
+
+      std::string buffer{};
+
+      // write_json returns a status code: false means success, true means error
+      expect(not glz::write_beve(v, buffer)) << "Failed to serialize async_vector";
+      glz::async_vector<int> result;
+      expect(not glz::read_beve(result, buffer)) << "Failed to deserialize async_vector";
+
+      auto result_reader = result.read();
+      expect(result_reader->size() == 5);
+      expect((*result_reader)[0] == 1);
+      expect((*result_reader)[1] == 2);
+      expect((*result_reader)[2] == 3);
+      expect((*result_reader)[3] == 4);
+      expect((*result_reader)[4] == 5);
+   };
+
+   "async_vector custom object serialization"_test = [] {
+      glz::async_vector<Point> points;
+      points.push_back({1, 2});
+      points.push_back({3, 4});
+      points.push_back({5, 6});
+
+      std::string buffer{};
+      expect(not glz::write_beve(points, buffer)) << "Failed to serialize custom objects in async_vector";
+
+      glz::async_vector<Point> result;
+      expect(not glz::read_beve(result, buffer)) << "Failed to deserialize custom objects in async_vector";
+
+      auto result_reader = result.read();
+      expect(result_reader->size() == 3);
+      expect((*result_reader)[0] == Point{1, 2});
+      expect((*result_reader)[1] == Point{3, 4});
+      expect((*result_reader)[2] == Point{5, 6});
+   };
+};
+
 int main() {}
